###########################################################################
 #   Copyright (C) 2006-2009 by Guy Rutenberg                              #
 #   http://www.guyrutenberg.com/contact-me/                               #
 #                                                                         #
 #   This program is free software; you can redistribute it and/or modify  #
 #   it under the terms of the GNU General Public License as published by  #
 #   the Free Software Foundation; either version 2 of the License, or     #
 #   (at your option) any later version.                                   #
 #                                                                         #
 #   This program is distributed in the hope that it will be useful,       #
 #   but WITHOUT ANY WARRANTY; without even the implied warranty of        #
 #   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         #
 #   GNU General Public License for more details.                          #
 #                                                                         #
 #   You should have received a copy of the GNU General Public License     #
 #   along with this program; if not, write to the                         #
 #   Free Software Foundation, Inc.,                                       #
 #   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             #
###########################################################################

import re
import sys


class CSSRtl:
    translators = []
    version = "2.0"
    def __init__(self, css_code):
        self.css_code = css_code
        self.enable_credit_line = True

    def credit_line(self):
        return "/* Translated to/from RTL using cssrtl.py-%s by Guy Rutenberg <http://www.guyrutenberg.com/contact-me> */\n" % self.version

    def strip_comments(self):
        """
        Strips the comments from the css file
        """
        strip_comments_reg = re.compile(r"(?:^|\*/)(.*?)(?:$|/\*)", re.DOTALL)
        matches = strip_comments_reg.findall(self.css_code)
        self.css_code_stripped = ''.join(matches)

    def get_css_rules(self):
        self.strip_comments()
        css_rules_reg = re.compile(r"(?P<selector>[^;{}]+){(?P<block>.*?)}", re.DOTALL)
        return css_rules_reg.findall(self.css_code_stripped)

    def parse_css_block(self, block):
        """
        parses given css block into statements
        """
        for rule in block.split(';'):
            if ':' in rule:
                prop, val = rule.split(':')
                yield prop.strip(), val.strip()

    def patch(self):
        patch = []
        if self.enable_credit_line:
            patch.append(self.credit_line())
        for selector, block in self.get_css_rules():
            block_translated = []
            selector = selector.strip()
            for prop, val in self.parse_css_block(block):
                for translator in self.translators:
                    try:
                        translated = translator(prop, val)
                    except ValueError:
                        print >> sys.stderr, "Couldn't translate rule %s: %s; for %s" %(prop, val, selector)
                        break
                    except CannotTranslateRule:
                        pass
                    else:
                        if translated.rule:
                            block_translated.append('\t'+translated.rule)
                        if translated.comment:
                            comment_lines = map(lambda x: "\t/* cssrtl: %s */" % x, translated.comment.splitlines())
                            block_translated += comment_lines
                        break # no need to translate the rule twice
            if block_translated:
                patch.append(selector+' {')
                patch += block_translated
                patch.append('}\n')
        return '\n'.join(patch)


class CannotTranslateRule(Exception):
    pass

class TranslatedRule:
    """
    Defines an object to hold data relevenet to a translated rule.
    """
    def __init__(self, rule="", comment=""):
        self.rule = rule
        self.comment = comment

def register_translator(func):
    """
    Registers a translation function. Translation function should accept a
    CSS property and value and return a TranslatedRule object. If the rule
    can't be translated by the translation function, a CannotTranslateRule
    exception should be raised.
    """
    CSSRtl.translators.append(func)
    return func

@register_translator
def translate_padding_margin(prop, val):
    if prop in ("margin", "padding"):
        quartet_reg = re.compile(r"(\d+\S+)\s*(\d+\S+)\s*(\d+\S+)\s*(\d+\S+)")
        m = quartet_reg.match(val)
        if m:
            return TranslatedRule("%s: %s %s %s %s;" % (prop, m.group(1), m.group(4), m.group(3), m.group(2)))
    raise CannotTranslateRule

@register_translator
def translate_left_right_property(prop, val):
    t = TranslatedRule()
    if "right" in prop and prop!="right":
        t.rule = "%s: %s;" % (prop.replace("right","left"), val)
        t.comment = "You should reset the '%s' to whatever '%s' was" % (prop, prop.replace("right", "left"))
        return t
    if "left" in prop and prop!="left":
        t.rule = "%s: %s;" % (prop.replace("left","right"), val)
        t.comment = "You should reset the '%s' to whatever '%s' was" % (prop, prop.replace("left", "right"))
        return t
    raise CannotTranslateRule

@register_translator
def translate_left_right_property_reset(prop, val):
    if prop=="right":
        return TranslatedRule("right: auto; %s: %s;" % ("left", val))
    if prop=="left":
        return TranslatedRule("left: auto; %s: %s;" % ("right", val))
    raise CannotTranslateRule

        
@register_translator
def translate_text_align_float(prop, val):
    if prop in ("float", "text-align", "clear"):
        if val=="right":
            return TranslatedRule("%s: left;" % prop)
        elif val=="left":
            return TranslatedRule("%s: right;" % prop)
    raise CannotTranslateRule

@register_translator
def translate_direction(prop, val):
    if prop=="direction":
        if val=="rtl":
            return TranslatedRule("direction: ltr;")
        elif val=="ltr":
            return TranslatedRule("direction: rtl;")
    raise CannotTranslateRule

@register_translator
def translate_background_position(prop, val):
    if prop=="background-position":
        m  = re.match(r"right\s*(\S*)", val)
        if m:
            return TranslatedRule("background-position: left %s;" % (m.group(1),))

        m  = re.match(r"left\s*(\S*)", val)
        if m:
            return TranslatedRule("background-position: right %s;" % (m.group(1),))

        # check if it uses percentage
        m  = re.match(r"(\S+)%\s*(\S*)", val)
        if m:
            return TranslatedRule("background-position: %f%% %s;" % (100-float(m.group(1)), m.group(2)))

        raise ValueError, "Cannot translate absolute length background position"
    raise CannotTranslateRule

@register_translator
def translate_background_position(prop, val):
    if prop=="background":
        m  = re.match(r".*\s+right\s*(\S*)", val)
        if m:
            return TranslatedRule("background-position: left %s;" % (m.group(1),))

        m  = re.match(r".*\s+left\s*(\S*)", val)
        if m:
            return TranslatedRule("background-position: right %s;" % (m.group(1),))
    raise CannotTranslateRule

@register_translator
def _font_family_comment(prop, val):
    if prop=="font-family":
        t = TranslatedRule()
        t.comment = "You may need to adjust the font family property\n"
        t.comment += "%s: %s;" % (prop, val)
        return t
    raise CannotTranslateRule
        


if __name__=="__main__":
    code = sys.stdin.read()
    c = CSSRtl(code)
    print c.patch()

